
(*
 * Copyright (C) 2007-2014 The laby team
 * You have permission to copy, modify, and redistribute under the
 * terms of the GPL-3.0. For full license terms, see gpl-3.0.txt.
 *)

val project_name : string

val version_string : string
val version_current : string
val version_base : string
val version_status : string

val build_system : string
val build_arch : string
val build_ocaml : string
val build_lablgtk : string
val build_lablgtk_sourceview : string
